package com.bpcl.lpg.core.service.impl;

import java.util.Objects;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.bpcl.lpg.facades.data.LpgPaymentRequestData;
import com.bpcl.lpg.core.model.LpgPaymentInfoModel;
import com.bpcl.lpg.core.credentials.LPGCredentialsService;
import com.bpcl.lpg.core.pojo.LpgPaymentResponseData;
import java.net.http.HttpResponse;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import com.bpcl.lpg.core.service.LpgPayAndBookConnectionService;
import com.bpcl.lpg.core.encryption.LpgEncryptionService;
import com.bpcl.lpg.core.service.FetchPaymentStatusService;

public class LpgFetchPaymentStatusServiceImpl implements FetchPaymentStatusService
{
    private LpgEncryptionService lpgEncryptionService;
    private LpgPayAndBookConnectionService lpgPayAndBookConnectionService;
    private Converter<HttpResponse<String>, LpgPaymentResponseData> lpgPayBookResponseDataConverter;
    private LPGCredentialsService lpgCredentialsService;
    
    public LpgPaymentResponseData fetchPaymentStatus(final LpgPaymentInfoModel paymentInfo) throws JsonProcessingException {
        final LpgPaymentRequestData requestData = this.prepareRequestData(paymentInfo);
        final HttpResponse<String> httpResponse = this.getLpgPayAndBookConnectionService().getPayAndBookResponse(requestData, "", "fetchPaymentStatus");
        return (LpgPaymentResponseData)this.getLpgPayBookResponseDataConverter().convert((Object)httpResponse);
    }
    
    private LpgPaymentRequestData prepareRequestData(final LpgPaymentInfoModel paymentInfo) {
        final LpgPaymentRequestData requestData = new LpgPaymentRequestData();
        requestData.setPartner_Trans_ID(paymentInfo.getTransactionID());
        if (Objects.nonNull(paymentInfo.getAccount())) {
            requestData.setLPG_ID(paymentInfo.getAccount().getLpgId());
        }
        requestData.setEndPoint("fetchPaymentStatus");
        requestData.setDist_Ref_No(paymentInfo.getTransactionID());
        requestData.setBPCL_Unique_ID("");
        requestData.setAPI_Req_No("25");
        requestData.setAddl1("");
        requestData.setAddl2("");
        requestData.setAddl3("");
        requestData.setHashValue(this.prepareHashValue(requestData));
        return requestData;
    }
    
    private String prepareHashValue(final LpgPaymentRequestData requestData) {
        final StringBuilder sb = new StringBuilder();
        final String endPointName = requestData.getEndPoint();
        final String partnerCode = this.lpgCredentialsService.getPartnerCodeByApiName(endPointName);
        final String secret = this.lpgCredentialsService.getPartnerSecretByApiName(endPointName);
        requestData.setPartner_Code(partnerCode);
        sb.append(partnerCode);
        sb.append(requestData.getPartner_Trans_ID());
        sb.append(requestData.getDist_Ref_No());
        sb.append(requestData.getLPG_ID());
        sb.append(requestData.getBPCL_Unique_ID());
        sb.append(requestData.getAPI_Req_No());
        sb.append(requestData.getAddl1());
        sb.append(requestData.getAddl2());
        sb.append(requestData.getAddl3());
        sb.append(secret);
        return this.getLpgEncryptionService().encodeBySHA512(sb.toString());
    }
    
    public LpgEncryptionService getLpgEncryptionService() {
        return this.lpgEncryptionService;
    }
    
    public void setLpgEncryptionService(final LpgEncryptionService lpgEncryptionService) {
        this.lpgEncryptionService = lpgEncryptionService;
    }
    
    public LpgPayAndBookConnectionService getLpgPayAndBookConnectionService() {
        return this.lpgPayAndBookConnectionService;
    }
    
    public void setLpgPayAndBookConnectionService(final LpgPayAndBookConnectionService lpgPayAndBookConnectionService) {
        this.lpgPayAndBookConnectionService = lpgPayAndBookConnectionService;
    }
    
    public Converter<HttpResponse<String>, LpgPaymentResponseData> getLpgPayBookResponseDataConverter() {
        return this.lpgPayBookResponseDataConverter;
    }
    
    public void setLpgPayBookResponseDataConverter(final Converter<HttpResponse<String>, LpgPaymentResponseData> lpgPayBookResponseDataConverter) {
        this.lpgPayBookResponseDataConverter = lpgPayBookResponseDataConverter;
    }
    
    public void setLpgCredentialsService(final LPGCredentialsService lpgCredentialsService) {
        this.lpgCredentialsService = lpgCredentialsService;
    }
}
